/*
 * modbus-tcp - buffer.c
 *	version 0.1.0
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <buffer.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

int buffer_init(struct buffer *b)
{
	b->data = NULL;
	b->size = 0;
	b->length = 0;
	b->current = 0;
	return 0;
}

int __buffer_alloc(struct buffer *b, size_t size)
{
	b->data = realloc(b->data, size);
	if ((b->data == NULL) && size)
		return -ENOMEM;
	b->size = size;
	return 0;
}

int buffer_alloc(struct buffer *b, size_t size)
{
	if (b->size && (size < b->size * 2))
		size = b->size * 2;
	return __buffer_alloc(b, size);
}

int buffer_skip(struct buffer *b, size_t count)
{
	ssize_t ret;

	if (b->length + count > b->size) {
		ret = buffer_alloc(b, b->length + count);
		if (ret)
			return ret;
	}
	memset((char *)b->data + b->length, 0, count);
	b->length += count;
	return 0;
}

ssize_t buffer_read(int fd, struct buffer *b, size_t count)
{
	ssize_t ret;

	if (b->length + count > b->size) {
		ret = buffer_alloc(b, b->length + count);
		if (ret)
			return ret;
	}
	ret = read(fd, (char *)b->data + b->length, count);
	if (ret > 0)
		b->length += ret;
	return ret;
}

ssize_t buffer_write(int fd, struct buffer *b)
{
	ssize_t ret;
	size_t to_write = b->length - b->current;

	if (!to_write)
		return 0;

	ret = write(fd, (char *)b->data + b->current, to_write);
	if (ret > 0)
		b->current += ret;
	return ret;
}

int buffer_completed(struct buffer *b)
{
	return b->length == b->current;
}

/**
 * buffer_destroy - destroy buffer
 * @b: buffer to destroy
 */
int buffer_destroy(struct buffer *b)
{
	free(b->data);
	return 0;
}
