/*
 * modbus-utils - compiler.h
 *	version 0.1.0
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __COMPILER_H
#define __COMPILER_H

#include <unistd.h>

#define BUG() \
	do { \
		fprintf(stderr, "BUG at %s:%d\n", __FILE__, __LINE__); \
		abort(); \
	} while (0)

#define BUG_ON(cond) \
	do { \
		if ((cond)) \
			BUG(); \
	} while (0)

#define likely(x)	(x)
#define unlikely(x)	(x)

/*
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:	the pointer to the member.
 * @type:	the type of the container struct this is embedded in.
 * @member:	the name of the member within the struct.
 */
#define container_of(ptr, type, member) ({			\
	typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)( (char *)__mptr - offsetof(type,member) );})

extern int verbose_level;

extern struct modbus_context modbus;

#define get_unaligned(ptr)						\
	({								\
		const struct __str {					\
			typeof(*(ptr)) __v;				\
	 	} __attribute__((packed)) *__p = (const struct __str *) \
	 		(ptr);						\
		__p->__v;						\
	})

#define put_unaligned(val, ptr)						\
	do {								\
		struct __str {						\
			typeof(*(ptr)) __v;				\
		} __attribute__((packed)) *__p = (struct __str *) (ptr);\
		__p->__v = (val);					\
	} while(0)

#if defined(__HP_cc) && (__HP_cc >= 61000)
#define NORETURN __attribute__((noreturn))
#define NORETURN_PTR
#elif defined(__GNUC__) && !defined(NO_NORETURN)
#define NORETURN __attribute__((__noreturn__))
#define NORETURN_PTR __attribute__((__noreturn__))
#elif defined(_MSC_VER)
#define NORETURN __declspec(noreturn)
#define NORETURN_PTR
#else
#define NORETURN
#define NORETURN_PTR
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#define __packed __attribute__((packed))

#endif
