/*
 * modbus-ctl
 *
 * Copyright (C) 2007  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <unistd.h>
#include <errno.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <modbus.h>
#include <wrapper.h>

static const char *dev = "/dev/modbus0";
static int new_bus = -1;
static int new_busaddr = -1;
static int new_retry_count = -1;

static void parse_command_line(int argc, char **args)
{
	int c;
	int option_index = 0;

	static struct option long_options[] = {
		{"bus", 1, 0, 'b'},
		{"busaddr", 1, 0, 'a'},
		{"device", 1, 0, 'd'},
		{"retry-count", 1, 0, 'r'},
		{0, 0, 0, 0},
	};

	while (1) {
		c = getopt_long(argc, args, "a:b:d:r:", long_options,
				&option_index);
		if (c == -1)
			break;
		switch (c) {
		case 'a':
			new_busaddr = atoi(optarg);
			break;
		case 'b':
			new_bus = atoi(optarg);
			break;
		case 'd':
			dev = optarg;
			break;
		case 'r':
			new_retry_count = atoi(optarg);
			break;
		default:
			fprintf(stderr, "Unknown option.\n");
			exit(1);
		}
	}
	if (optind < argc) {
		if (optind < argc)
			fprintf(stderr, "warning: unknown parameters "
					"detected.\n");
	}
}

#define unlikely(x)	(x)

int main(int argc, char **args)
{
	struct modbus_context ctx;
	int ret;

	parse_command_line(argc, args);

	ret = modbus_init(&ctx);
	if (ret)
		die("modbus_init: %s\n", strerror(-ret));
	ret = modbus_open(&ctx, dev);
	if (ret)
		die("modbus_init: %s\n", strerror(-ret));

	if (new_bus != -1) {
		ret = modbus_set_bus(&ctx, new_bus);
		if (ret)
			die("modbus_set_bus() error: %s\n", strerror(-ret));
	}

	if (new_busaddr != -1) {
		ret = modbus_set_device(&ctx, new_busaddr);
		if (ret)
			die("modbus_set_device() error: %s\n", strerror(-ret));
	}
	if (new_retry_count != -1) {
		ret = modbus_set_retry(&ctx, new_retry_count);
		if (ret)
			die("modbus_set_retry() error: %s\n", strerror(-ret));
	}
	return 0;
}
