/*
 * modbus-utils - modbus.h
 *	version 0.1.0
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __MODBUS_H
#define __MODBUS_H

#include <stddef.h>
#include <list.h>
#include <inttypes.h>

#define MODBUS_ADU_SIZE_MAX	253

struct mbap_header {
	uint16_t seq;
	uint16_t proto;
	uint16_t len;
	uint8_t addr;
} __attribute__((packed));
#define MBAP_MTU (sizeof(struct mbap_header) + MODBUS_ADU_SIZE_MAX)

#define MBEX_NOSYS		0x01
#define MBEX_FAULT		0x02
#define MBEX_INVAL		0x03
#define MBEX_SLAVE_FAIL		0x04
#define MBEX_ACK		0x05
#define MBEX_BUSY		0x06
#define MBEX_MEMPARERR		0x08
#define MBEX_NOGW		0x0a
#define MBEX_GWERR		0x0b

/* size of kernel Modbus header */
#define MODBUS_KHDR_SIZE	4

struct modbus_context;

struct modbus_request {
	unsigned int id;
	struct modbus_context *ctx;
	void *c;
	size_t c_len;
	void *r;
	size_t r_len;
	struct list_head list;
};

struct modbus_context {
	struct list_head reqs;
	int fd;
	int emu;	/* packed mode emulation */
};

struct modbus_addr {
	unsigned int bus;
	unsigned int addr;
};
#define MODBUS_ADDR_NULL	{ 0, 0 }

int modbus_name_to_addr(struct modbus_addr *addr, const char *name);
int modbus_addr_to_name(struct modbus_addr *addr, char **name);

int modbus_init(struct modbus_context *ctx);
int modbus_set_emu(struct modbus_context *ctx, int emu);
int modbus_open(struct modbus_context *ctx, const char *dev);
int modbus_set_packet(struct modbus_context *ctx);
int modbus_set_bus(struct modbus_context *ctx, unsigned int bus);
int modbus_set_device(struct modbus_context *ctx, unsigned int device);
int modbus_set_retry(struct modbus_context *ctx, unsigned int retry);

/* packet interface */
int modbus_send(struct modbus_context *ctx, struct modbus_request *re);
int modbus_poll(struct modbus_context *ctx, struct modbus_request **r,
		int timeout);
int modbus_process(struct modbus_context *ctx, struct modbus_request **re,
		int events);

/* block interface */
int modbus_read(struct modbus_context *ctx, struct modbus_addr *addr,
		void *buf, unsigned int len, unsigned int offset);

int modbus_close(struct modbus_context *ctx);
int modbus_destroy(struct modbus_context *ctx);

int modbus_request_init(struct modbus_request *r);
int modbus_request_destroy(struct modbus_request *r);
int modbus_request_fill_id(struct modbus_request *r);

/* defined by user */
void *xmalloc(size_t count);

#endif
