/*
 * pu-buff - buffer management
 *
 * Copyright (C) 2011..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef PU_BUFF_H_INCLUDED
#define PU_BUFF_H_INCLUDED

#include <list.h>
#include <stddef.h>

typedef unsigned int pu_gfp_t;

struct pu_buff {
	struct list_head list;

	unsigned int tail;
	unsigned int end;
	unsigned int len;
	unsigned int data_len;
	unsigned char *data;
	unsigned char *head;

	unsigned int users;
};

#ifndef GFP_ATOMIC
#define GFP_ATOMIC 0
#endif

#ifndef GFP_KERNEL
#define GFP_KERNEL 0
#endif

struct pu_buff *pu_alloc_skb(unsigned int size, pu_gfp_t priority);
void pu_consume_skb(struct pu_buff *skb);
struct pu_buff *pu_skb_get(struct pu_buff *skb);
#define pu_free_skb(skb) pu_consume_skb(skb)
void pu_skb_reserve(struct pu_buff *skb, int len);

struct pu_buff *skb_copy_expand(struct pu_buff *pu, int newheadroom,
		int newtailroom, pu_gfp_t priority);

unsigned char *skb_tail_pointer(const struct pu_buff *skb);
unsigned char *pu_skb_tail_pointer(const struct pu_buff *skb);
void pu_skb_reset_tail_pointer(struct pu_buff *skb);
void pu_skb_set_tail_pointer(struct pu_buff *skb, const int offset);

unsigned int pu_skb_headroom(const struct pu_buff *skb);
unsigned int pu_skb_tailroom(const struct pu_buff *skb);
void pu_skb_trim(struct pu_buff *skb, unsigned int len);
unsigned char *pu_skb_put(struct pu_buff *skb, unsigned int len);
unsigned char *pu_skb_push(struct pu_buff *skb, unsigned int len);
unsigned char *pu_skb_pull(struct pu_buff *skb, unsigned int len);

struct pu_buff *pu_skb_copy(struct pu_buff *src);

void pu_dump_skb(struct pu_buff *skb);

#endif
