/*
 * modbus-utils - wrapper.h
 *
 * Copyright (C) 2012..2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifndef PU_LIB_H_INCLUDED
#define PU_LIB_H_INCLUDED

#include <stdlib.h>
#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif
#include <compiler.h>

#ifndef LOG_CRIT
#define	LOG_CRIT	2	/* critical conditions */
#endif
#ifndef LOG_ERR
#define	LOG_ERR		3	/* error conditions */
#endif
#ifndef LOG_WARNING
#define	LOG_WARNING	4	/* warning conditions */
#endif
#ifndef LOG_NOTICE
#define	LOG_NOTICE	5	/* normal but significant condition */
#endif
#ifndef LOG_INFO
#define	LOG_INFO	6	/* informational */
#endif
#ifndef LOG_DEBUG
#define	LOG_DEBUG	7	/* debug-level messages */
#endif

void logger_init(void);
void log_printk(int priority, const char *fmt, ...);
#define log_crit(fmt...)	log_printk(LOG_CRIT, fmt)
#define log_error(fmt...)	log_printk(LOG_ERR, fmt)
#define log_warn(fmt...)	log_printk(LOG_WARNING, fmt)
#define log_info(fmt...)	log_printk(LOG_INFO, fmt)
#define log_debug(fmt...)	log_printk(LOG_DEBUG, fmt)

#ifdef HAVE_HTONL
#include <netinet/in.h>

#define cpu_to_be16(x)		(htons(x))
#define cpu_to_be32(x)		(htonl(x))
#define be16_to_cpu(x)		(ntohs(x))
#define be32_to_cpu(x)		(ntohl(x))
#else

#ifdef HAVE_ENDIAN_H
#include <endian.h>
#endif

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#endif

#if BYTE_ORDER == BIG_ENDIAN
# define cpu_to_be16(x)		(x)
# define cpu_to_be32(x)		(x)
# define be16_to_cpu(x)		(x)
# define be32_to_cpu(x)		(x)
#elif BYTE_ORDER == LITTLE_ENDIAN
# define cpu_to_be16(x)		(bswap_16(x))
# define cpu_to_be32(x)		(bswap_32(x))
# define be16_to_cpu(x)		(bswap_16(x))
# define be32_to_cpu(x)		(bswap_32(x))
#else
#error "please specify byte order"
#endif

#endif

#ifndef PATH_MAX
#define PATH_MAX	4096
#endif

void *xmalloc(size_t size);
void xfree(const void *ptr);
ssize_t xread(int fd, void *buf, size_t len);
ssize_t xwrite(int fd, const void *buf, size_t len);
ssize_t write_in_full(int fd, const void *buf, size_t count);
ssize_t read_in_full(int fd, void *buf, size_t count);
void NORETURN die(const char *err, ...);
void NORETURN die_errno(const char *fmt, ...);
void write_or_die(int fd, const void *buf, size_t count);

void warn(const char *fmt, ...);
void info(const char *fmt, ...);

#endif
